import os
from ase.io import read
from ase.ga.data import PrepareDB
from ase import Atoms
from ase.ga.startgenerator import StartGenerator
from ase.ga.utilities import closest_distances_generator
from ase.ga.utilities import get_all_atom_types
from ase.constraints import FixAtoms
import numpy as np
from ase.build import fcc111


db_file = 'gadb.db'
ref_file = 'ref.db'

# Create the box

#slab = fcc111('Au', size=(4, 4, 1), vacuum=10.0, orthogonal=True)
#slab.set_constraint(FixAtoms(mask=len(slab) * [True]))

#L = 10 #box length
#cluster = Atoms('AlAlAl', positions=[(0.2*L,0.2*L,0.2*L),(0.2*L,0.2*L,0.45*L),(0.2*L,0.45*L,0.2*L)], cell = (L,L,L), pbc = [1, 1, 1])
#cluster.set_constraint(FixAtoms(mask=len(cluster) * [True]))
#slab = cluster

zeolite = read('OHAlfa1Min.xyz', format ='xyz')
zeolite.set_constraint(FixAtoms(mask=len(zeolite) * [True]))
slab = zeolite

# Define the volume in which the adsorbed cluster is optimized
# the volume is defined by a corner position (p0)
# and three spanning vectors (v1, v2, v3)
p0 = np.array([5.0 , 6.5 , 7.0 ])
v1 = np.array( [0.  , 0. , 1. ]) *  4.5
v2 = np.array( [0.  , 1. , 0. ]) *  4.5
v3 = np.array( [1.  , 0. , 0. ]) *  7.0 


# Define the composition of the atoms to optimize
atom_numbers =  2 * [31] + 3 * [8] + 2 * [1]

# Define the closest distance two atoms of a given species can be to each other
unique_atom_types = get_all_atom_types(slab, atom_numbers)
cd = closest_distances_generator(atom_numbers=unique_atom_types,
                                 ratio_of_covalent_radii=0.7)

# Create the starting population
sg = StartGenerator(slab=slab,
                    atom_numbers=atom_numbers,
                    closest_allowed_distances=cd,
                    box_to_place_in=[p0, [v1, v2, v3]])

# Generate the starting population
population_size = 20
starting_population = [sg.get_new_candidate() for i in range(population_size)]

#from ase.visualize import view   # uncomment these lines
#view(starting_population)        # to see the starting population

# Create the database to store information in
d = PrepareDB(db_file_name=db_file,
              simulation_cell=slab,
              population_size=population_size,
              stoichiometry=atom_numbers)

# Create the reference database to store reference data in if no ref database is present
if not os.path.exists(ref_file):
    r = PrepareDB(db_file_name=ref_file, 
              simulation_cell=slab,
              population_size=population_size,
              stoichiometry=atom_numbers) 


for a in starting_population:
    d.add_unrelaxed_candidate(a)
